@@ -1,17 +1,17 @@
-use auto_ops::*;
+use auto_ops::impl_op_ex;
 
 use crate::{Quaternion, Vec3, Vec4};
 
 /// A struct representing a 4x4 matrix.
-/// 
+///
 /// It's values are stored in column-major order by default,
 /// as expected by OpenGL and accepted by all other APIs.
 /// To change this, use feature `mat-row-major`
 #[derive(Debug, Clone, Copy, PartialEq)]
-#[cfg_attr(feature="serde", derive(serde::Serialize, serde::Deserialize))]
+#[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
 #[repr(C)]
 pub struct Mat4 {
-    pub values: [f32; 4*4],
+    pub values: [f32; 4 * 4],
 }
 
 impl Default for Mat4 {
@@ -21,11 +21,11 @@ impl Default for Mat4 {
     }
 }
 
-#[cfg(not(feature="mat-row-major"))]
+#[cfg(not(feature = "mat-row-major"))]
 const fn cr(c: usize, r: usize) -> usize {
     r + c * 4
 }
-#[cfg(feature="mat-row-major")]
+#[cfg(feature = "mat-row-major")]
 const fn cr(c: usize, r: usize) -> usize {
     r * 4 + c
 }
@@ -35,11 +35,8 @@ impl Mat4 {
     pub const fn identity() -> Self {
         Self {
             values: [
-                1.0, 0.0, 0.0, 0.0,
-                0.0, 1.0, 0.0, 0.0,
-                0.0, 0.0, 1.0, 0.0,
-                0.0, 0.0, 0.0, 1.0,
-            ]
+                1.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 1.0,
+            ],
         }
     }
 
@@ -50,7 +47,7 @@ impl Mat4 {
         res.values[cr(3, 0)] = t.x;
         res.values[cr(3, 1)] = t.y;
         res.values[cr(3, 2)] = t.z;
-        
+
         res
     }
 
@@ -89,19 +86,19 @@ impl Mat4 {
     }
 
     /// Creates a 3D local-to-world/object-to-world matrix.
-    /// 
+    ///
     /// When multiplying this matrix by a vector, it will be
     /// - scaled by `s`
     /// - rotated by `r`
     /// - translated by `t`
-    /// 
+    ///
     /// in this order.
     pub fn local_to_world(t: Vec3, r: Quaternion, s: Vec3) -> Self {
         Self::translate(t) * Self::rotate(r) * Self::scale(s)
     }
 
     /// Creates a 3D world-to-local/world-to-object matrix.
-    /// 
+    ///
     /// This matrix does the opposite of [`local_to_world()`](Self::local_to_world())
     pub fn world_to_local(t: Vec3, r: Quaternion, s: Vec3) -> Self {
         Self::scale(1.0 / s) * Self::rotate(-r) * Self::translate(-t)
@@ -109,7 +106,14 @@ impl Mat4 {
 
     /// Creates an orthographic projection matrix
     /// with z mapped to \[0; 1\], as expected by Vulkan.
-    pub fn orthographic_vulkan(left: f32, right: f32, bottom: f32, top: f32, near: f32, far: f32) -> Self {
+    pub fn orthographic_vulkan(
+        left: f32,
+        right: f32,
+        bottom: f32,
+        top: f32,
+        near: f32,
+        far: f32,
+    ) -> Self {
         let mut res = Self::identity();
 
         let a = 2.0 / (right - left);
@@ -133,7 +137,14 @@ impl Mat4 {
 
     /// Creates an orthographic projection matrix
     /// with z mapped to \[-1; 1\], as expected by OpenGL.
-    pub fn orthographic_opengl(left: f32, right: f32, bottom: f32, top: f32, near: f32, far: f32) -> Self {
+    pub fn orthographic_opengl(
+        left: f32,
+        right: f32,
+        bottom: f32,
+        top: f32,
+        near: f32,
+        far: f32,
+    ) -> Self {
         let mut res = Self::identity();
 
         let a = 2.0 / (right - left);
@@ -157,10 +168,17 @@ impl Mat4 {
 
     /// Creates an inverse orthographics projection matrix
     /// with z mapped to \[0; 1\], as expected by Vulkan.
-    /// 
+    ///
     /// The world position of a uv/depth pair can be reconstructed with the same code as shown
     /// in [`inverse_perspective_vulkan()`](Self::inverse_perspective_vulkan())
-    pub fn inverse_orthographic_vulkan(left: f32, right: f32, bottom: f32, top: f32, near: f32, far: f32) -> Self {
+    pub fn inverse_orthographic_vulkan(
+        left: f32,
+        right: f32,
+        bottom: f32,
+        top: f32,
+        near: f32,
+        far: f32,
+    ) -> Self {
         let mut res = Self::identity();
 
         let a = 2.0 * (right - left);
@@ -172,7 +190,7 @@ impl Mat4 {
 
         res.values[cr(0, 0)] = 1.0 / a;
         res.values[cr(3, 0)] = -b / a;
-        
+
         res.values[cr(1, 1)] = 1.0 / c;
         res.values[cr(3, 1)] = -d / c;
 
@@ -184,10 +202,17 @@ impl Mat4 {
 
     /// Creates an inverse orthographics projection matrix
     /// with z mapped to \[-1; 1\], as expected by OpenGL.
-    /// 
+    ///
     /// The world position of a uv/depth pair can be reconstructed with the same code as shown
     /// in [`inverse_perspective_opengl()`](Self::inverse_perspective_opengl())
-    pub fn inverse_orthographic_opengl(left: f32, right: f32, bottom: f32, top: f32, near: f32, far: f32) -> Self {
+    pub fn inverse_orthographic_opengl(
+        left: f32,
+        right: f32,
+        bottom: f32,
+        top: f32,
+        near: f32,
+        far: f32,
+    ) -> Self {
         let mut res = Self::identity();
 
         let a = 2.0 * (right - left);
@@ -199,7 +224,7 @@ impl Mat4 {
 
         res.values[cr(0, 0)] = 1.0 / a;
         res.values[cr(3, 0)] = -b / a;
-        
+
         res.values[cr(1, 1)] = 1.0 / c;
         res.values[cr(3, 1)] = -d / c;
 
@@ -217,7 +242,7 @@ impl Mat4 {
 
         res.values[cr(0, 0)] = 1.0 / (thfov * aspect);
         res.values[cr(1, 1)] = 1.0 / thfov;
-        
+
         res.values[cr(2, 2)] = far / (far - near);
         res.values[cr(3, 2)] = (-far * near) / (far - near);
 
@@ -235,7 +260,7 @@ impl Mat4 {
 
         res.values[cr(0, 0)] = 1.0 / (thfov * aspect);
         res.values[cr(1, 1)] = 1.0 / thfov;
-        
+
         res.values[cr(2, 2)] = (far + near) / (far - near);
         res.values[cr(3, 2)] = (-2.0 * far * near) / (far - near);
 
@@ -247,7 +272,7 @@ impl Mat4 {
 
     /// Creates an inverse perspective matrix
     /// with z mapped to \[0; 1\], as expected by Vulkan.
-    /// 
+    ///
     /// This matrix can be used to reconstruct the world position from a uv/depth pair in a shader.
     /// This pseudo code can be used:
     /// ```GLSL
@@ -276,7 +301,7 @@ impl Mat4 {
 
     /// Creates an inverse perspective matrix
     /// with z mapped to \[-1; 1\], as expected by OpenGL.
-    /// 
+    ///
     /// This matrix can be used to reconstruct the world position from a uv/depth pair in a shader.
     /// This pseudo code can be used:
     /// ```GLSL
@@ -314,6 +339,7 @@ impl Mat4 {
     }
 
     /// Returns a transposed copy of `self`.
+    #[must_use]
     pub fn transposed(&self) -> Mat4 {
         let mut res = Mat4::identity();
 
@@ -347,32 +373,43 @@ impl Mat4 {
     }
 }
 
-impl_op_ex!(* |a: &Mat4, b: &Mat4| -> Mat4 {
+impl_op_ex!(*|a: &Mat4, b: &Mat4| -> Mat4 {
     let mut res = Mat4::identity();
 
     for r in 0..4 {
         for c in 0..4 {
-            res.values[cr(c, r)] = 
-                a.values[cr(0, r)] * b.values[cr(c, 0)] +
-                a.values[cr(1, r)] * b.values[cr(c, 1)] +
-                a.values[cr(2, r)] * b.values[cr(c, 2)] +
-                a.values[cr(3, r)] * b.values[cr(c, 3)];
+            res.values[cr(c, r)] = a.values[cr(0, r)] * b.values[cr(c, 0)]
+                + a.values[cr(1, r)] * b.values[cr(c, 1)]
+                + a.values[cr(2, r)] * b.values[cr(c, 2)]
+                + a.values[cr(3, r)] * b.values[cr(c, 3)];
         }
     }
 
     res
 });
 
-impl_op_ex!(* |a: &Mat4, b: &Vec4| -> Vec4 {
+impl_op_ex!(*|a: &Mat4, b: &Vec4| -> Vec4 {
     Vec4 {
-        x: a.values[cr(0, 0)] * b.x + a.values[cr(1, 0)] * b.y + a.values[cr(2, 0)] * b.z + a.values[cr(3, 0)] * b.w,
-        y: a.values[cr(0, 1)] * b.x + a.values[cr(1, 1)] * b.y + a.values[cr(2, 1)] * b.z + a.values[cr(3, 1)] * b.w,
-        z: a.values[cr(0, 2)] * b.x + a.values[cr(1, 2)] * b.y + a.values[cr(2, 2)] * b.z + a.values[cr(3, 2)] * b.w,
-        w: a.values[cr(0, 3)] * b.x + a.values[cr(1, 3)] * b.y + a.values[cr(2, 3)] * b.z + a.values[cr(3, 3)] * b.w,
+        x: a.values[cr(0, 0)] * b.x
+            + a.values[cr(1, 0)] * b.y
+            + a.values[cr(2, 0)] * b.z
+            + a.values[cr(3, 0)] * b.w,
+        y: a.values[cr(0, 1)] * b.x
+            + a.values[cr(1, 1)] * b.y
+            + a.values[cr(2, 1)] * b.z
+            + a.values[cr(3, 1)] * b.w,
+        z: a.values[cr(0, 2)] * b.x
+            + a.values[cr(1, 2)] * b.y
+            + a.values[cr(2, 2)] * b.z
+            + a.values[cr(3, 2)] * b.w,
+        w: a.values[cr(0, 3)] * b.x
+            + a.values[cr(1, 3)] * b.y
+            + a.values[cr(2, 3)] * b.z
+            + a.values[cr(3, 3)] * b.w,
     }
 });
 
-impl_op_ex!(* |a: &Mat4, b: &Vec3| -> Vec3 {
+impl_op_ex!(*|a: &Mat4, b: &Vec3| -> Vec3 {
     Vec3 {
         x: a.values[cr(0, 0)] * b.x + a.values[cr(1, 0)] * b.y + a.values[cr(2, 0)] * b.z,
         y: a.values[cr(0, 1)] * b.x + a.values[cr(1, 1)] * b.y + a.values[cr(2, 1)] * b.z,
@@ -382,18 +419,18 @@ impl_op_ex!(* |a: &Mat4, b: &Vec3| -> Vec3 {
 
 impl From<[f32; 16]> for Mat4 {
     fn from(d: [f32; 16]) -> Self {
-        Self{values: d}
+        Self { values: d }
     }
 }
 
 impl From<[[f32; 4]; 4]> for Mat4 {
     fn from(d: [[f32; 4]; 4]) -> Self {
-        Self{values: [
-            d[0][0], d[0][1], d[0][2], d[0][3],
-            d[1][0], d[1][1], d[1][2], d[1][3],
-            d[2][0], d[2][1], d[2][2], d[2][3],
-            d[3][0], d[3][1], d[3][2], d[3][3],
-        ]}
+        Self {
+            values: [
+                d[0][0], d[0][1], d[0][2], d[0][3], d[1][0], d[1][1], d[1][2], d[1][3], d[2][0],
+                d[2][1], d[2][2], d[2][3], d[3][0], d[3][1], d[3][2], d[3][3],
+            ],
+        }
     }
 }
 
